@@ -399,6 +399,46 @@ on_widget_leave(GtkWidget* widget, GdkEventCrossing *event, gpointer data)
     return FALSE;
 }
 
+
+#ifdef DEBUG
+static void
+debuglog_close(GpsDataLoggerWidget * self)
+{
+    if (self->logfile) {
+	fclose(self->logfile);
+	self->logfile = NULL;
+    }
+}
+
+
+static void
+debuglog_start(GpsDataLoggerWidget * self)
+{
+    gchar *path;
+    time_t current;
+    struct tm *tp;
+
+
+    /* If the logfile is already open (should not) then close it */
+    if (self->logfile)
+        debuglog_close(self);
+    /* Name the log file using the current time */
+    current = time(NULL);
+    tp = localtime(&current);
+    path = g_strdup_printf("%s/%d%02d%02d_%02d%02d%02d.log", 
+		           self->settings->log_folder,
+		    	   tp->tm_year + 1900,
+		   	   tp->tm_mon + 1,
+		   	   tp->tm_mday,
+		   	   tp->tm_hour,
+		   	   tp->tm_min,
+		   	   tp->tm_sec );
+    self->logfile = fopen(path, "w");
+    g_free(path);
+    
+}
+#endif
+
 static void
 logfile_close(GpsDataLoggerWidget * self)
 {
@@ -469,6 +509,13 @@ logfile_start(GpsDataLoggerWidget * self)
 static void
 update_status_and_buttons (GpsDataLoggerWidget * self)
 {
+#ifdef DEBUG
+    if (self->logfile) {
+        g_fprintf(self->logfile, "%s -- enter (state: %d)\n", __func__, self->app_state);
+	fflush(self->logfile);
+    }
+#endif
+
     switch(self->app_state) {
 	case APP_STATE_STARTING:
 		gtk_image_set_from_pixbuf (GTK_IMAGE(self->gps_status), 
@@ -509,6 +556,12 @@ update_status_and_buttons (GpsDataLoggerWidget * self)
 		break;
     }
     gtk_widget_queue_draw(GTK_WIDGET(self));
+#ifdef DEBUG
+    if (self->logfile) {
+        g_fprintf(self->logfile, "%s -- leave\n", __func__);
+	fflush(self->logfile);
+    }
+#endif
 }
 
 static void
@@ -516,6 +569,12 @@ handle_button_press (GtkWidget *button,
 		     gpointer data)
 {
     GpsDataLoggerWidget *self = GPS_DATA_LOGGER_WIDGET (data);
+#ifdef DEBUG
+    if (self->logfile) {
+        g_fprintf(self->logfile, "%s -- enter (state: %d)\n", __func__, self->app_state);
+	fflush(self->logfile);
+    }
+#endif
 
     if (button == self->button_rec) {
         if (self->app_state == APP_STATE_IDLE) {
@@ -525,7 +584,9 @@ handle_button_press (GtkWidget *button,
             g_print("handle_button_press : calling update_status_and_buttons\n");
 #endif
             update_status_and_buttons(self);
-	    location_gps_device_reset_last_known(self->device);
+	    /* ToDo: verify that this is the function that stucks
+	     * and causes the widget not to respond anymore */
+	    /* location_gps_device_reset_last_known(self->device); */
             location_gpsd_control_start(self->control);
 	    /* open a new log file */
 	    logfile_start(self);
@@ -565,48 +626,154 @@ handle_button_press (GtkWidget *button,
 	/* close log file */
 	logfile_close(self);
     }
+#ifdef DEBUG
+    if (self->logfile) {
+        g_fprintf(self->logfile, "%s -- leave (new state: %d)\n", __func__, self->app_state);
+	fflush(self->logfile);
+    }
+#endif
 }
 
 static gboolean
 on_button_release(GtkWidget* widget, GdkEventButton *button_event, gpointer data)
 {
     GpsDataLoggerWidget *self = GPS_DATA_LOGGER_WIDGET (data);
+    gboolean retval = FALSE;
 
+#ifdef DEBUG
+    if (self->logfile) {
+        g_fprintf(self->logfile, "%s -- enter ", __func__);
+        if (widget == GTK_WIDGET(self->background)) {
+	    g_fprintf(self->logfile, " widget: background\n");
+	} else if (widget == GTK_WIDGET(self->button_rec)) {
+	    g_fprintf(self->logfile, " widget: button_rec\n");
+	} else if (widget == GTK_WIDGET(self->button_stop)) {
+	    g_fprintf(self->logfile, " widget: button_stop\n");
+	}
+	fflush(self->logfile);
+    }
+#endif
     self->rec_button_down = FALSE;
     self->stop_button_down = FALSE;
+    gtk_widget_queue_draw(GTK_WIDGET(self));
 
-    if ((widget == GTK_WIDGET(self->button_rec)) && self->rec_button_enabled)
-        {
-	handle_button_press(self->button_rec, self);
+    if (widget == GTK_WIDGET(self->background)) {
+        if (button_event->x >= self->button_rec->allocation.x &&
+            button_event->y >= self->button_rec->allocation.y &&
+            button_event->x <= self->button_rec->allocation.x +
+                           self->button_rec->allocation.width &&
+            button_event->y <= self->button_rec->allocation.y +
+                           self->button_rec->allocation.height &&
+            self->rec_button_enabled) {
+#ifdef DEBUG
+                if (self->logfile) {
+                    g_fprintf(self->logfile, "  calling handle_button_press on background for button_rec\n");
+	            fflush(self->logfile);
+		}
+#endif
+                handle_button_press(self->button_rec, self);
+        } else if (button_event->x >= self->button_stop->allocation.x &&
+            button_event->y >= self->button_stop->allocation.y &&
+            button_event->x <= self->button_stop->allocation.x +
+                           self->button_stop->allocation.width &&
+            button_event->y <= self->button_stop->allocation.y +
+                           self->button_stop->allocation.height &&
+            self->stop_button_enabled) {
+#ifdef DEBUG
+                if (self->logfile) {
+                    g_fprintf(self->logfile, "  calling handle_button_press on background for button_stop\n");
+	            fflush(self->logfile);
+		}
+#endif
+                handle_button_press(self->button_stop, self);
+        }	
+    } else if ((widget == GTK_WIDGET(self->button_rec)) && 
+		    self->rec_button_enabled) {
+#ifdef DEBUG
+        if (self->logfile) {
+            g_fprintf(self->logfile, "  calling handle_button_press on button_rec for button_rec\n");
+	    fflush(self->logfile);
 	}
-    else if ((widget == GTK_WIDGET(self->button_stop)) && self->stop_button_enabled)
-        {
+#endif
+        handle_button_press(self->button_rec, self);
+    } else if ((widget == GTK_WIDGET(self->button_stop)) && 
+		    self->stop_button_enabled) {
+#ifdef DEBUG
+        if (self->logfile) {
+             g_fprintf(self->logfile, "  calling handle_button_press on button_stop for button_stop\n");
+	     fflush(self->logfile);
+	}
+#endif
         handle_button_press(self->button_stop, self);
-	}
-
-    return FALSE;
+    }
+    
+#ifdef DEBUG
+    if (self->logfile) {
+        g_fprintf(self->logfile, "%s -- leave\n", __func__);
+	fflush(self->logfile);
+    }
+#endif
+    return retval;
 }
 
 static gboolean
 on_button_press(GtkWidget* widget, GdkEventButton *button_event, gpointer data)
 {
     GpsDataLoggerWidget *self = GPS_DATA_LOGGER_WIDGET (data);
+    gboolean retval = FALSE;
 
-    if ((widget == GTK_WIDGET(self->button_rec)) &&
-        self->rec_button_enabled)
-    {
+#ifdef DEBUG
+    if (self->logfile) {
+        g_fprintf(self->logfile, "%s -- enter ", __func__);
+        if (widget == GTK_WIDGET(self->background)) {
+	    g_fprintf(self->logfile, " widget: background\n");
+	} else if (widget == GTK_WIDGET(self->button_rec)) {
+	    g_fprintf(self->logfile, " widget: button_rec\n");
+	} else if (widget == GTK_WIDGET(self->button_stop)) {
+	    g_fprintf(self->logfile, " widget: button_stop\n");
+	}
+	fflush(self->logfile);
+    }
+#endif
+    if (widget == GTK_WIDGET(self->background)) {
+        if (button_event->x >= self->button_rec->allocation.x &&
+           button_event->y >= self->button_rec->allocation.y &&
+            button_event->x <= self->button_rec->allocation.x +
+                           self->button_rec->allocation.width &&
+            button_event->y <= self->button_rec->allocation.y +
+                           self->button_rec->allocation.height &&
+            self->rec_button_enabled)
+        {
+            self->rec_button_down = TRUE;
+            gtk_widget_queue_draw(widget);
+        } else if (button_event->x >= self->button_stop->allocation.x &&
+            button_event->y >= self->button_stop->allocation.y &&
+            button_event->x <= self->button_stop->allocation.x +
+                           self->button_stop->allocation.width &&
+            button_event->y <= self->button_stop->allocation.y +
+                           self->button_stop->allocation.height &&
+            self->stop_button_enabled)
+        {
+            self->stop_button_down = TRUE;
+            gtk_widget_queue_draw(widget);
+        }
+    } else if ((widget == GTK_WIDGET(self->button_rec)) &&
+		    self->rec_button_enabled) {
         self->rec_button_down = TRUE;
         gtk_widget_queue_draw(widget);
-    }
-    else if ((widget == GTK_WIDGET(self->button_stop)) &&
-        self->stop_button_enabled)
-    {
+    } else if ((widget == GTK_WIDGET(self->button_stop)) &&
+		    self->stop_button_enabled) {
         self->stop_button_down = TRUE;
         gtk_widget_queue_draw(widget);
     }
 
-    return FALSE;
-
+#ifdef DEBUG
+    if (self->logfile) {
+        g_fprintf(self->logfile, "%s -- leave\n", __func__);
+	fflush(self->logfile);
+    }
+#endif
+    return retval;
 }
 
 static void
@@ -615,6 +782,12 @@ on_error(LocationGPSDControl *control, gpointer data)
     GpsDataLoggerWidget *self = GPS_DATA_LOGGER_WIDGET (data);
 #ifndef __arm__
     g_printf("%s\n", __func__);
+#endif
+#ifdef DEBUG
+    if (self->logfile) {
+        g_fprintf(self->logfile, "%s -- enter\n", __func__);
+	fflush(self->logfile);
+    }
 #endif
     if (self->app_state != APP_STATE_IDLE) {
 	hildon_banner_show_information(NULL,
@@ -940,6 +1113,9 @@ gps_data_logger_widget_init (GpsDataLoggerWidget * self)
     GdkColormap *colormap;
 
     self->file = NULL;
+#ifdef DEBUG
+    self->logfile = NULL;
+#endif
     self->app_state = APP_STATE_IDLE;
     /* Preload all icon pixmaps */
     load_icons(self);
@@ -1029,13 +1205,22 @@ gps_data_logger_widget_init (GpsDataLoggerWidget * self)
 		    self);
 
     /* button signals */
+
     g_signal_connect(G_OBJECT(self->button_rec), "button-release-event",
-                    G_CALLBACK(on_button_release), self);
+		    G_CALLBACK(on_button_release), self);
     g_signal_connect(G_OBJECT(self->button_rec), "button-press-event",
-                    G_CALLBACK(on_button_press), self);
+		    G_CALLBACK(on_button_press), self);
     g_signal_connect(G_OBJECT(self->button_stop), "button-release-event",
-                    G_CALLBACK(on_button_release), self);
+		    G_CALLBACK(on_button_release), self);
     g_signal_connect(G_OBJECT(self->button_stop), "button-press-event",
+		    G_CALLBACK(on_button_press), self);
+    /* On the first release of Fremantle all the button events are
+     * going to the background, so let's register those too
+     * ToDo: figure out how to check for the updated sw version
+     *       and not register button signals for the background there */
+    g_signal_connect(G_OBJECT(self->background), "button-release-event",
+                    G_CALLBACK(on_button_release), self);
+    g_signal_connect(G_OBJECT(self->background), "button-press-event",
                     G_CALLBACK(on_button_press), self);
     g_signal_connect(G_OBJECT(self->background), "leave-notify-event",
                     G_CALLBACK(on_widget_leave), self);
@@ -1051,6 +1236,9 @@ gps_data_logger_widget_init (GpsDataLoggerWidget * self)
                      (gpointer)self);
     g_signal_connect(G_OBJECT(self->background), "style-set",
                      G_CALLBACK(on_style_set), (gpointer)self);
+#ifdef DEBUG
+    debuglog_start(self);
+#endif
 }
 
 static void
@@ -1059,6 +1247,9 @@ gps_data_logger_widget_finalize (GObject *object)
     GpsDataLoggerWidget *self = GPS_DATA_LOGGER_WIDGET (object);
     /* Close the logfile if it is open */
     logfile_close(self);
+#ifdef DEBUG
+    debuglog_close(self);
+#endif
     g_object_unref(self->device);
     g_object_unref(self->control);
     if (self->settings_dialog) {
