@@ -56,6 +56,9 @@ struct _GpsDataLoggerWidget
 	LocationGPSDControl *control;
 	LocationGPSDevice *device;
 	FILE		*file;
+#ifdef DEBUG
+	FILE		*logfile;
+#endif
 	GtkWidget	*settings_dialog;
 	GpsDataLoggerSettings *settings;
 	/* Settings dialog widgets */
